from pydantic import BaseModel, EmailStr,validator
from typing import List, IO, Union, Any
from fastapi import UploadFile
import  os
from mimetypes import MimeTypes

class MessageSchema(BaseModel):
    receipients: List[EmailStr]
    attachments: List[Any] = []
    subject: str = ""
    body: str = None
    cc: List[EmailStr] = []
    bcc: List[EmailStr] = []
    charset: str = "utf-8"

    @validator("attachments")
    def validate_file(cls,v):
        temp = []
        for file in v:
            if isinstance(file,str):

                if os.path.isfile(file) and os.access(file, os.R_OK):
                    # mime = magic.Magic(mime=True)
                    mime = MimeTypes()
                    mime_type = mime.guess_type(file)
                    with open(file,mode="rb") as f:
                        u = UploadFile(f.name,f.read(),content_type=mime_type[0])
                        temp.append(u)
                else:
                    raise  ValueError("incorrect file path for attachment or not readable")
            elif isinstance(file,UploadFile):
                temp.append(file)
            else:
                raise  ValueError("attachments field type incorrect, must be UploadFile or path")
        return temp





import asyncio
import time

from datetime import date

from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart

from email.utils import formatdate, make_msgid

from fastapi_mail.version import PY3
from email.encoders import encode_base64


class MailMsg:
    """
    Preaparation of class for email text
    :param subject: email subject header
    :param recipients: list of email addresses
    :param body: plain text message
    :param html: HTML message
    :param sender: email sender address
    :param cc: CC list
    :param bcc: BCC list
    :param attachments: list of Attachment instances
    """

    def __init__(self, **entries):
        self.__dict__.update(entries)
        self.msgId = make_msgid()


    def _mimetext(self, text, subtype="plain"):
        """Creates a MIMEText object"""
        return MIMEText(text, _subtype=subtype, _charset=self.charset)


    async def attach_file(self, message, attachment):

        print(attachment)
        
        for file in attachment:
        
            part = MIMEBase(*file.content_type.split('/'))

            
            part.set_payload(file.file)
            encode_base64(part)

            filename = file.filename

            try:
                filename and filename.encode('ascii')
            except UnicodeEncodeError:
                if not PY3:
                    filename = filename.encode('utf8')

            filename = ('UTF8', '', filename)
            

            part.add_header(
                'Content-Disposition',
                "attachment",
                filename=filename)
            

            self.message.attach(part)


    
    async def _message(self,sender):
        """Creates the email message"""
            
        self.message = MIMEMultipart()
        self.message.set_charset(self.charset)
        self.message['Date'] = formatdate(time.time(), localtime=True)
        self.message['Message-ID'] = self.msgId
        self.message["To"] =  ', '.join(self.receipients)
        self.message["From"] = sender


        if self.subject:
            self.message["Subject"] = (self.subject)
           
        if self.cc:
            self.message["Cc"] = ', '.join(self.cc)
        
        if self.bcc:
            self.message["Bcc"] = ', '.join(self.bcc)

        if self.body:
            self.message.attach(self._mimetext(self.body))

        if self.attachments:
            await self.attach_file(self.message, self.attachments)


        return self.message

    
    async def as_string(self):
        return await self._message().as_string()
        
    def as_bytes(self):
        return self._message().as_bytes()

    def __str__(self):
        return self.as_string()

    def __bytes__(self):
        return self.as_bytes()





from fastapi_mail.config import  ConnectionConfig
from fastapi_mail.errors import ConnectionErrors, PydanticClassRequired
from pydantic import BaseSettings  as Settings
import aiosmtplib



class Connection:
    '''
    Manages Connection to provided email service with its credentials
    '''

    def __init__(self,settings: ConnectionConfig ):

        if not issubclass(settings.__class__,Settings):
            raise  PydanticClassRequired('''Email configuruation should be provided with ConnectionConfig class, check example below:
         \nfrom fastmail.config import ConnectionConfig \nclass ConnectionConfig: \nMAIL_USERNAME:  EmailStr =None\nMAIL_PASSWORD: str =None \nMAIL_PORT: int  = 25 \nMAIL_SERVER: str = '127.0.0.1' \nMAIL_TLS: bool = False \nMAIL_SSL: bool = False \nconnection = Connection(ConnectionConfig)
         ''')

        self.settings = settings.dict()


    async def __aenter__(self): #setting up a connection
        await self._configure_connection()
        return self

    async def __aexit__(self, exc_type, exc, tb): #closing the connection
        await self.session.quit()


    async def _configure_connection(self):
        try:
            self.session = aiosmtplib.SMTP(
                hostname = self.settings.get("MAIL_SERVER"), 
                port = self.settings.get("MAIL_PORT"), 
                use_tls= self.settings.get("MAIL_SSL"),
                start_tls = self.settings.get("MAIL_TLS")
               )
            await self.session.connect()

            await self.session.login(
                self.settings.get("MAIL_USERNAME"), 
                self.settings.get("MAIL_PASSWORD")
                )
           
        except Exception as error:
            raise ConnectionErrors(f"Exception raised {error}, check your credentials or email service configuration") 



class NotAvilableService(Exception):
    def __init__(self, expression, message):
        self.expression = expression
        self.message = message


class TypeExecption(TypeError):

    def __init__(self, expression):
        self.expression = expression

class ConnectionErrors(Exception):

    def __init__(self, expression):
        self.expression = expression


class WrongPort(Exception):

    def __init__(self, expression, message):
        self.expression = expression
        self.message = message

class WrongFormat(Exception):

    def __init__(self, expression, message):
        self.expression = expression
        self.message = message

class WrongFile(Exception):

    def __init__(self, expression, message):
        self.expression = expression
        self.message = message


class PydanticClassRequired(Exception):

    def __init__(self, expression):
        self.expression = expression




from fastapi_mail.fastmail import FastMail
from fastapi_mail.version import VERSION
from fastapi_mail.config import  ConnectionConfig
from fastapi_mail.schemas import MessageSchema


__author__ = "sabuhi.shukurov@gmail.com"




__all__ = [
    "FastMail", "VERSION", "ConnectionConfig", "MessageSchema", 
]


from fastapi_mail.config import  ConnectionConfig
from fastapi_mail.connection import Connection
from fastapi_mail.schemas import MessageSchema
from fastapi_mail.msg import MailMsg
import asyncio
import aiosmtplib


class FastMail:
    '''
    Fastapi mail system sending mails(individual, bulk) attachments(individual, bulk)

    '''
    def __init__(self,
        config : ConnectionConfig
        ):

        self.config = config

    async def __preape_message(self, message):
        msg = MailMsg(**message.dict())
        return await msg._message(self.config.MAIL_USERNAME)

    async def send_message(self, message: MessageSchema):
        msg = await self.__preape_message(message)

        async with Connection(self.config) as session:
            await session.session.send_message(msg)





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="fastapi-mail",
    version=__import__("fastapi_mail").VERSION,
    author="Sabuhi Shukurov",
    author_email="sabuhi.shukurov@gmail.com",
    description="Simple lightwigh mail sending for FastApi",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url="https://github.com/sabuhish/fastapi-mail",

     install_requires=[
        "fastapi>=0.61.1",
        "aiosmtplib>=1.1.4",
        "python-multipart==0.0.5",
        "pydantic>=1.6.1",
        "email-validator>=1.1.1"
    ],
    # install_requires=["fastapi","aiosmtplib","python-multipart", "pydantic","pydantic[email]", "email-validator"],
    # extras_require={
    #     "fastapi": ["fastapi"],
    #     "pydantic": ["pydantic"],
    #     "aiosmtplib": ["aiosmtplib"],
    #     "python-multipart": ["python-multipart"],
    #     "email_validator": ["email-validator"]

    # },
    platforms=['any'],
    packages=setuptools.find_packages(),
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from pydantic import BaseSettings as Settings

from pydantic import EmailStr


class ConnectionConfig(Settings):

    MAIL_USERNAME:  EmailStr 
    MAIL_PASSWORD: str 
    MAIL_PORT: int  = 465
    MAIL_SERVER: str 
    MAIL_TLS: bool = False
    MAIL_SSL: bool = True
    MAIL_DEBUG: int = 1


